<?php
namespace MyMenu\Controllers;

use DateTime;

use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Teknisa\Libs\Util\Utilities;

use MyMenu\Services\JWT;
use MyMenu\Libraries\KeyValueStorage;
use MyMenu\Util\RequestHeader;

class MyMenuSession
{

    private $crudService;
    private $environment;
    private $sessionManager;
    private $fileServerURL;
    private $customerID;
    private $privacyPolicyLink;

    public function __construct($crudService, $environment, $sessionManager, $fileServerURL, $customerID, $privacyPolicyLink)
    {
        $this->crudService = $crudService;
        $this->environment = $environment;
        $this->sessionManager = $sessionManager;
        $this->fileServerURL = $fileServerURL;
        $this->customerID = $customerID;
        $this->privacyPolicyLink = $privacyPolicyLink;
    }

    public function login($request, $response)
    {
        $userData = $request->getRow();

        $user = $userData['USER'];
        $passwd = $userData['PW'];
        $md5Passwd = md5($passwd);

        $conditions = array(
            array(
                "columnName" => "UPPER_DSEMAILUSUARIOAPP",
                "operator" => "=",
                "value" => strtoupper($user),
            ),
            array(
                "columnName" => "DSSENHAUSUARIOAPP",
                "operator" => "=",
                "value" => $md5Passwd,
            ),
            array(
                "columnName" => "NRORG",
                "operator" => "=",
                "value" => KeyValueStorage::get("NRORG"),
            ),
        );

        $resDataSet = $this->crudService->getFilteredDataSet('usuarioapp_login', $conditions, 1, 1);

        if (sizeof($resDataSet->getRows()) == 1) {
            $userData = $resDataSet->getRows()[0];

            $isNewUser = $userData['USUARIOATIVO'] === null;
            if ($userData['USUARIOATIVO']=='S' || $isNewUser) {
                $sessionData = $this->initializeSessionData($userData);
                $this->manageAssociation($userData, $request);
                $response->addDataSet(new DataSet('user', $sessionData["userData"]));
                $response->addDataSet(new DataSet('JWT', [$sessionData["token"]]));
            } else {
                $response->setError(new Error("Usuário Inativo", 500));
            }
        } else {
            $response->setError(new Error("Dados de login inválidos", 500));
        }
    }

    public function anonimousAccount($request, $response)
    {
        $conditions = array(
            array(
                "columnName" => "UPPER_DSEMAILUSUARIOAPP",
                "operator" => "=",
                "value" => strtoupper('anonimo@anonimo.com'),
            ),
            array(
                "columnName" => "NRORG",
                "operator" => "=",
                "value" => KeyValueStorage::get("NRORG"),
            ),
        );

        $resDataSet = $this->crudService->getFilteredDataSet('usuarioapp_login', $conditions, 1, 1);
        if (sizeof($resDataSet->getRows()) == 1) {
            $userData = $resDataSet->getRows()[0];
            $userData['isAnon'] = true;
            $sessionData = $this->initializeSessionData($userData);
            $response->addDataSet(new DataSet('user', $sessionData["userData"]));
            $response->addDataSet(new DataSet('JWT', [$sessionData["token"]]));
        } else {
            $row = array(
                '__is_new' => true,
                '__new_code_pk' => true,
                '__cast_number' => true,
                '__new_code_pk' => 'USUARIOAPP',
                '__table_pk' => 'NRUSUARIOAPP',
                'DSEMAILUSUARIOAPP' => 'anonimo@anonimo.com',
                'DSSENHAUSUARIOAPP' => null,
                'DSTELEFONE' => 'null',
                'DSPRIMEIRONOME' => 'Convidado',
                'DSULTIMONOME' => 'Anônimo',
                'FACEBOOK_ID' => null,
                'FACEBOOK_TOKEN' => null,
                'FACEBOOK_TOKEN_EXP_DATE' => null,
                'FOTO' => null,
                'CDVERIFICASENHA' => null,
            );

            $dataSetToSave = new DataSet('usuarioapp', array($row));
            $saveRequest = new DTO\Request\DataSet($dataSetToSave, $request->getMethod(), $request->getRoutePath(), $request->getUserId());
            $rowsToSave = $this->crudService->getRowsToSave($saveRequest, 'usuarioapp');
            try {
                $this->crudService->save('usuarioapp', $rowsToSave);
                $resDataSet = $this->crudService->getFilteredDataSet('usuarioapp_login', $conditions, 1, 1);
                if (sizeof($resDataSet->getRows()) == 1) {
                    $userData = $resDataSet->getRows()[0];
                    $userData['isAnon'] = true;
                    $sessionData = $this->initializeSessionData($userData);
                    $this->manageAssociation($userData, $request);
                    $response->addDataSet(new DataSet('user', $sessionData["userData"]));
                    $response->addDataSet(new DataSet('JWT', [$sessionData["token"]]));
                } else {
                    return false;
                }
            } catch (\Exception $e) {
                $uniqueErr = 'ORA-00001';
                $errorMsg = $e->getMessage();
                if (substr_count($errorMsg, $uniqueErr)) {
                    $errorMsg = 'O modo anónimo deve estar desabilitado.';
                }
                $response->setError(new Error($errorMsg, 500));
            }
        }
    }

    public function logout($request, $response)
    {
        KeyValueStorage::remove("NRUSUARIOAPP");
        KeyValueStorage::remove("DSEMAILUSUARIOAPP");
        //TODO require JWT removal
    }

    public function validateSession($request, $response)
    {
        $token = RequestHeader::getKey("token", null);
        if ($token) {
            $validToken = JWT::validateToken($token, 'mymenuapp');
            $tokenData = JWT::getTokenData($token);
            if ($validToken && isset($tokenData['NUMQRCODE']) && isset($tokenData['NRORG']) && isset($tokenData['CDFILIAL']) && isset($tokenData["NRUSUARIOAPP"])) {
                $arr = array(
                    "NUMQRCODE" => $tokenData['NUMQRCODE'],
                    "NRORG" => $tokenData['NRORG'],
                    "CDFILIAL" => $tokenData['CDFILIAL'],
                    "NRLOCALPESQ" => isset($tokenData['NRLOCALPESQ']) ? $tokenData['NRLOCALPESQ'] : null,
                    "NRUSUARIOAPP" => $tokenData["NRUSUARIOAPP"],
                    "DSEMAILUSUARIOAPP" => $tokenData["DSEMAILUSUARIOAPP"],
                );

                KeyValueStorage::set("CDFILIAL", $tokenData['CDFILIAL']);
                KeyValueStorage::set("NRLOCALPESQ", $tokenData['NRLOCALPESQ']);
                KeyValueStorage::set("NRORG", $tokenData['NRORG']);

                KeyValueStorage::set("NRUSUARIOAPP", $tokenData["NRUSUARIOAPP"]);
                KeyValueStorage::set("DSEMAILUSUARIOAPP", $tokenData["DSEMAILUSUARIOAPP"]);

                $this->environment->setNrOrg($tokenData['NRORG']);
                $this->environment->setNrOrgTrab($tokenData['NRORG']);
                $this->environment->setCdOperador($tokenData["NRUSUARIOAPP"]);

                $this->environment->setCdFilial(KeyValueStorage::get("CDFILIAL"));
                $this->environment->setNrOrg(KeyValueStorage::get("NRORG"));
                $this->environment->setNrOrgTrab(KeyValueStorage::get("NRORG"));
                

                $token = JWT::generateToken($arr);
                $response->addDataSet(new DataSet('JWT', [$token]));
                
                return true;

            } else {
                $response->setError(new Error("Sessão inválida__", 500));
            }
        } else {
            $response->setError(new Error("Sessão inválida_", 500));
        }
    }

    public function validateQrCode($request, $response)
    {
        $token = RequestHeader::getKey("token", null);
        if ($token) {
            $validToken = JWT::validateToken($token, 'mymenuapp');
            $tokenData = JWT::getTokenData($token);
            if ($validToken && isset($tokenData['NUMQRCODE']) && isset($tokenData['NRORG']) && isset($tokenData['CDFILIAL'])) {
                if(!$tokenData['NUMQRCODE'] || !$tokenData['NRORG'] || !$tokenData['CDFILIAL']){
                    $this->readQRCode($request, $response);
                    return;
                }
                $arr = array(
                    "NUMQRCODE" => $tokenData['NUMQRCODE'],
                    "NRORG" => $tokenData['NRORG'],
                    "CDFILIAL" => $tokenData['CDFILIAL'],
                    "NRLOCALPESQ" => $tokenData['NRLOCALPESQ'],
                );
                $this->environment->setNrOrg($tokenData['NRORG']);
                $this->environment->setNrOrgTrab($tokenData['NRORG']);

                $token = JWT::generateToken($arr);
                $response->addDataSet(new DataSet('JWT', [$token]));
                return true;
            } else {
                $this->readQRCode($request, $response);
            }
        } else {
            $this->readQRCode($request, $response);
        }
    }
    
    public function getDecriptedUrlData($request, $response)
    {
        
        $criptedUrlData = $request->getRow();
        $cript = $criptedUrlData['criptedData'];
        
        $decriptedData = json_decode(Utilities::zeedhiDecrypt($cript));
        
        $response->addDataSet(new DataSet('decriptedData', [$decriptedData]));
    }
    
    public function readQRCode($request, $response)
    {

        $qrCodeData = $request->getRow();

        if (isset($qrCodeData['nrqrcode']) && isset($qrCodeData['nrorg'])) {
            $nrqrcode = $qrCodeData['nrqrcode'];
            $nrorg = $qrCodeData['nrorg'];

            $conditions = array(
                array(
                    "columnName" => "NRQRCODELOCALPESQ",
                    "operator" => "=",
                    "value" => $nrqrcode,
                ),
                array(
                    "columnName" => "NRORG",
                    "operator" => "=",
                    "value" => $nrorg,
                ),
            );
            $resDataSet = $this->crudService->getFilteredDataSet('qrcodelocalpesq', $conditions, 1, 1);

            if (sizeof($resDataSet->getRows()) == 1) {
                $qrCodeData = $resDataSet->getRows()[0];
                $cdfilial = $qrCodeData['CDFILIAL'];
                $nrlocalpesq = $qrCodeData['NRLOCALPESQ'];
                $urlLogo = $qrCodeData['URLLOGO'];
                $urlLGPD = $qrCodeData['DSARQPOLPRIV'] != null ? $qrCodeData['DSARQPOLPRIV'] : $this->privacyPolicyLink;
            
                $today = new DateTime();
                $dtexpiracao = new DateTime();

                if ($qrCodeData['DTEXPIRACAO']) {
                    $dtexpiracao = date_create_from_format('d/m/Y H:i:s', $qrCodeData['DTEXPIRACAO']);
                }
                
                if ($today > $dtexpiracao) {
                    $response->setError(new Error("MSG_EXPIRED_QRCODE", 500));
                } else {
                    $this->environment->setNrOrg($nrorg);
                    $this->environment->setNrOrgTrab($nrorg);

                    $token = JWT::generateToken(array(
                        "NUMQRCODE" => $nrqrcode,
                        "NRORG" => $nrorg,
                        "CDFILIAL" => $cdfilial,
                        "NRLOCALPESQ" => $nrlocalpesq,
                    ));
                    $response->addDataSet(new DataSet('JWT', [$token]));
                    
                    $response->addDataSet(new DataSet('fileServerURL', [$this->fileServerURL]));
                    $qrCodeData['CUSTOMERID'] = $this->customerID;
                    $response->addDataSet(new DataSet('qrcode', $qrCodeData));
                    $response->addDataSet(new DataSet('tosURL', [$urlLGPD]));
                    return true;
                }
            } else {
                $response->setError(new Error("QRCode inválido", 500));
            }
        } else {
            $response->setError(new Error("QRCode inválido", 500));
        }
    }
    
    public function localpesq($request, $response)
    {
        $qrCodeData = $request->getRow();
    }
    
    public function updateQrcodeConfig($request, $response){
        $qrCodeData = $request->getRow();

        if (isset($qrCodeData['nrqrcode']) && isset($qrCodeData['nrorg'])) {
            $nrqrcode = $qrCodeData['nrqrcode'];
            $nrorg = $qrCodeData['nrorg'];
            
            $conditions = array(
                array(
                    "columnName" => "NRQRCODELOCALPESQ",
                    "operator" => "=",
                    "value" => $nrqrcode,
                ),
                array(
                    "columnName" => "NRORG",
                    "operator" => "=",
                    "value" => $nrorg,
                ),
            );
            $resDataSet = $this->crudService->getFilteredDataSet('qrcodelocalpesq', $conditions, 1, 1);

            if (sizeof($resDataSet->getRows()) == 1) {
                $qrCodeData = $resDataSet->getRows()[0];
                $response->addDataSet(new DataSet('qrcode', $qrCodeData));
                return true;
            }

        }
    }

    public function setNRORGONSESSION($request, $response)
    {
        $qrCodeData = $request->getRow();
        if (isset($qrCodeData['nrqrcode']) && isset($qrCodeData['nrorg'])) {
            $nrqrcode = $qrCodeData['nrqrcode'];
            $nrorg = $qrCodeData['nrorg'];

            KeyValueStorage::set('nrorg', $nrorg);
            KeyValueStorage::set("NRORG", $nrorg);
            $this->environment->setNrOrg($nrorg);
            $this->environment->setNrOrgTrab($nrorg);

            JWT::unsetSessionCookie("mymenuapp");
            $token = JWT::generateToken(array(
                "NUMQRCODE" => $nrqrcode,
                "NRORG" => $nrorg,
            ));
            $response->addDataSet(new DataSet('JWT', [$token]));
            JWT::setSessionCookie("mymenuapp", $token, 0);
            return true;
        } else {
            $response->setError(new Error("QRCode inválido ou falha na rede!", 500));
        }
    }

    protected function initializeSessionData($userData)
    {
        $isAnon = isset($userData["isAnon"]) ? $userData["isAnon"] : false;

        KeyValueStorage::set("isAnon", $isAnon);
        KeyValueStorage::set("NRUSUARIOAPP", $userData["NRUSUARIOAPP"]);
        KeyValueStorage::set("DSEMAILUSUARIOAPP", $userData["DSEMAILUSUARIOAPP"]);

        $this->environment->setCdFilial(KeyValueStorage::get("CDFILIAL"));
        $this->environment->setCdOperador($userData["NRUSUARIOAPP"]);
        $this->environment->setNrOrg(KeyValueStorage::get("NRORG"));
        $this->environment->setNrOrgTrab(KeyValueStorage::get("NRORG"));

        $usr = array(
            "isAnon" => $isAnon,
            "NRUSUARIOAPP" => $userData["NRUSUARIOAPP"],
            "CDFILIAL" => KeyValueStorage::get("CDFILIAL"),
            "NRORG" => KeyValueStorage::get("NRORG"),
            "NAME" => $userData['DSPRIMEIRONOME'].' '.$userData['DSULTIMONOME']
        );

        $oldToken = RequestHeader::getKey("token");
        $token = $oldToken;
        $isValid = JWT::validateToken($oldToken, 'mymenuapp');
        if ($oldToken && $isValid) {
            $oldTokenData = JWT::getTokenData($oldToken);
            $arr = array(
                "NUMQRCODE" => $oldTokenData['NUMQRCODE'],
                "NRORG" => $oldTokenData['NRORG'],
                "CDFILIAL" => $oldTokenData['CDFILIAL'],
                "NRLOCALPESQ" => $oldTokenData['NRLOCALPESQ'],
                "NRUSUARIOAPP" => $userData["NRUSUARIOAPP"],
                "DSEMAILUSUARIOAPP" => $userData["DSEMAILUSUARIOAPP"],
                "isAnon" => $isAnon
            );
            $token = JWT::generateToken($arr);
        } else {
            throw new \Exception("Ocorreu um erro com o QrCode salvo, leia novamente!");
        }
        return [
            "userData" => $usr,
            "token" => $token
        ];
    }

    public function facebookAuth($request, $response)
    {

        $userData = $request->getRow();

        $conditions = array(
            array(
                "columnName" => "FACEBOOK_ID",
                "operator" => "=",
                "value" => $userData['userID'],
            ),
            array(
                "columnName" => "DSEMAILUSUARIOAPP",
                "operator" => "=",
                "value" => $userData['email'],
            ),
        );

        $resDataSet = $this->crudService->getFilteredDataSet('facebook_login', $conditions, 1, 1);
        if (sizeof($resDataSet->getRows()) == 1) {
            $rowsToSave = $resDataSet->getRows();
            $rowsToSave[0]['FACEBOOK_TOKEN_EXP_DATE'] = $userData['expiresIn'];
            $rowsToSave[0]['FACEBOOK_TOKEN'] = $userData['accessToken'];
            $res = $this->crudService->save('usuarioapp', $rowsToSave);
        } else {
            $saveRequest = $this->_getRequestForFacebookRegister($request);
            $rowsToSave = $this->crudService->getRowsToSave($saveRequest, 'usuarioapp');
            $this->crudService->save('usuarioapp', $rowsToSave);
            $resDataSet = $this->crudService->getFilteredDataSet('facebook_login', $conditions, 1, 1);
        }

        if(sizeof($resDataSet->getRows()) == 1 && 
        ($resDataSet->getRows()[0]['USUARIOATIVO']=='S' || 
        $resDataSet->getRows()[0]['USUARIOATIVO']==null)){
            $userData = $resDataSet->getRows()[0];
            $sessionData = $this->initializeSessionData($userData);
            $this->manageAssociation($userData, $request);
            $response->addDataSet(new DataSet('user', $sessionData["userData"]));
            $response->addDataSet(new DataSet('JWT', [$sessionData["token"]]));
        } else {
            $response->setError(new Error("Usuário Inativo", 500));
        }
    }

    private function _getRequestForFacebookRegister($request)
    {
        $accountData = $request->getRow();
        $row = array(
            '__is_new' => true,
            '__new_code_pk' => true,
            '__cast_number' => true,
            '__new_code_pk' => 'USUARIOAPP',
            '__table_pk' => 'NRUSUARIOAPP',
            'DSEMAILUSUARIOAPP' => $accountData['email'],
            'DSSENHAUSUARIOAPP' => null,
            'DSTELEFONE' => null, // maybe this can change in the future if facebook includes phone number in the graph api
            'DSPRIMEIRONOME' => $accountData['first_name'],
            'DSULTIMONOME' => $accountData['last_name'],
            'FACEBOOK_ID' => $accountData['userID'],
            'FACEBOOK_TOKEN' => $accountData['accessToken'],
            'FACEBOOK_TOKEN_EXP_DATE' => $accountData['expiresIn'],
            'FOTO' => null,
            'CDVERIFICASENHA' => null,
        );

        $dataSetToSave = new DataSet('usuarioapp', array($row));
        $saveRequest = new DTO\Request\DataSet($dataSetToSave, $request->getMethod(), $request->getRoutePath(), $request->getUserId());
        return $saveRequest;
    }

    private function getQrCodeNumberFromToken()
    {
        $token = RequestHeader::getKey("token", null);
        if ($token) {
            $tokenData = JWT::getTokenData($token);
            return $tokenData['NUMQRCODE'];
        }
        return null;
    }
    
    private function manageAssociation($userData, $request)
    {
        $conditions = array(
            array(
                "columnName" => "NRUSUARIOAPP",
                "operator" => "=",
                "value" => $userData['NRUSUARIOAPP'],
            ),
            array(
                "columnName" => "NRORG",
                "operator" => "=",
                "value" => KeyValueStorage::get("NRORG"),
            ),
        );
        $checkAssociation = $this->crudService->getFilteredDataSet('usuarioapp_usuarioaplicativo', $conditions, 1, 1);
        $QrCode = $this->getQrCodeNumberFromToken();

        if ($checkAssociation->getRows()[0]['ASSOCIATION'] != 1) {
            $this->createAssociation($userData, $QrCode, $request);
        } else {
            $this->updateAssociation($userData, $QrCode, $request);
        }

    }

    private function createAssociation($userData, $QrCode, $request)
    {
        $row = array(
            '__is_new' => true,
            '__new_code_pk' => true,
            '__cast_number' => true,
            '__new_code_pk' => 'APP_USUARIOAPLICATIVO',
            '__table_pk' => 'NRUSUARIOAPLICATIVO',
            'NRUSUARIOAPP' => $userData['NRUSUARIOAPP'],
            'NRORG' => KeyValueStorage::get("NRORG"),
            'NRQRCODELOCALPESQ' => $QrCode,
            'NRAPLICATIVO' => 1,
            'DTULTIMOLOGIN' => "",
        );
        $dataSetToSave = new DataSet('app_usuarioaplicativo', array($row));
        $saveRequest = new DTO\Request\DataSet($dataSetToSave, $request->getMethod(), $request->getRoutePath(), $request->getUserId());
        $rowsToSave = $this->crudService->getRowsToSave($saveRequest, 'app_usuarioaplicativo');
        $this->crudService->save('app_usuarioaplicativo', $rowsToSave);
    }

    private function updateAssociation($userData, $QrCode, $request)
    {
        $conditions = array(
            array(
                "columnName" => "NRUSUARIOAPP",
                "operator" => "=",
                "value" => $userData['NRUSUARIOAPP'],
            ),
            array(
                "columnName" => "NRORG",
                "operator" => "=",
                "value" => KeyValueStorage::get("NRORG"),
            ),
            array(
                "columnName" => "NRAPLICATIVO",
                "operator" => "=",
                "value" => 1,
            )
        );
        $resDataSet = $this->crudService->getFilteredDataSet('app_usuarioaplicativo', $conditions, 1, 1);
        if (sizeof($resDataSet->getRows()) == 1) {
            $updateRequest = new DTO\Request\DataSet($resDataSet, $request->getMethod(), $request->getRoutePath(), $request->getUserId());
            $rowsToUpdate = $this->crudService->getRowsToSave($updateRequest, 'app_usuarioaplicativo');
            if (sizeof($rowsToUpdate) == 1) {
                $rowsToUpdate[0]["NRQRCODELOCALPESQ"] = $QrCode;
                $rowsToUpdate[0]["DTULTIMOLOGIN"] = "";
                $this->crudService->save('app_usuarioaplicativo', $rowsToUpdate);
            }
        }
    }

    public function getCustomerID(DTO\Request\Row $request, DTO\Response $response) {
        $response->addDataSet(new DataSet('data', ['customerID' => $this->customerID]));
    }
    
    public function validateUser(DTO\Request\Row $request, DTO\Response $response) {
        $row = $request->getRow();
        $nrusuarioapp = $row['NRUSUARIOAPP'] ?? null;
        if( !$nrusuarioapp ) {
            throw new \Exception("ERR_INVALID_PARAMS");
        }
        $userCondition = [
            [
                "columnName" => "NRUSUARIOAPP",
                "operator"   => "=",
                "value"      => $nrusuarioapp 
            ]
        ];
        $dataSet = $this->crudService->getFilteredDataSet('usuarioapp_login', $userCondition, 1, 1);
        $users = $dataSet->getRows();
        $validations = [];
        if(is_array($users) && isset($users[0])) {
            $user = $users[0];
            $validations["IS_ACTIVE"] = $user["USUARIOATIVO"] === "S";
        }
        $response->addDataSet(new DataSet("USER_VALIDATION", $validations));
    }
}