function MasterDetailController($scope, ScreenService, templateManager, eventEngine) {

	var self = this;
	var CONTROLLER_KEY = 'masterDetailControllerInit';
	var WIDGET_AFTER_MOVE_ROW_EVENT_NAME = "WidgetAfterMoveRow";
	var WIDGET_ON_CLOSE_SWIPE_EVENT_NAME = "WidgetOnCloseSwipe";
	var WIDGET_ON_OPEN_DETAIL_EVENT_NAME = "WidgetOnOpenDetail";

	var eventsToCreate = ["afterMoveRow", "onNewRow", "onOpenDetail", "onCloseSwipe"];

	Object.defineProperty(this, "widget", {
		get : function(){
			return $scope.widget;
		},
		set : function (widget){
			$scope.widget = widget;
		},
		enumerable: true,
		configurable: true
	});

	var formTemplate = this.widget.templateToForm;
	var viewTemplate = this.widget.templateToView;

	this.widget.templateToForm = "widget/master_detail/tabbedWidget.html";
	this.widget.templateToView = "widget/master_detail/tabbedWidget.html";

	function createEvents(events) {
		events.forEach(function(event){
			if (!self.widget[event]) {
				eventEngine.createEvent(self.widget, event);
			}
		});
	}
	createEvents(eventsToCreate);

	var afterMoveRowBoundEventIndex = eventEngine.bindEvent(this.widget, WIDGET_AFTER_MOVE_ROW_EVENT_NAME, setCurrentRowAndOpenSwipe);

	function setCurrentRowAndOpenSwipe() {
		if (!self.widget.inlineEditing && !Util.isClickOnEditableField()) {
			var dataSource = self.widget.dataSource;
			if (dataSource) {
				if (!self.widget.lastCheckedRows) {
					self.widget.lastCheckedRows = 0;
				}
				if ((!dataSource.checkedRows || dataSource.checkedRows.length === 0) &&
					(!(self.widget.lastCheckedRows === 1 && dataSource.checkedRows.length === 0))) {
					self.widget.currentRow.__index = dataSource.data.indexOf(self.widget.currentRow);
					if (self.widget.selectDetailOnMoveRow) {
						self.widget.currentWidget = self.widget;
					}
					self.widget.view();
					self.widget.childTemplate = self.widget.defaultChildTemplate;

					self.widget.isMasterDetailPopup = true;
					if (!ScreenService.isWidgetAsSwipe(self.widget) || self.widget.closeChildSwipes) {
						ScreenService.openSwipe(self.widget).then(function() {
							if (self.widget.onOpenDetail) {
								self.widget.onOpenDetail();
							}
						});
					} else {
						if (self.widget.currentWidget) {
							self.widget.currentWidget.activate();
						}
					}
				}
				self.widget.lastCheckedRows = dataSource.checkedRows.length;
			}
		}
	}
	$scope.$on("$destroy", unbindAfterMoveRowListener);
	function unbindAfterMoveRowListener() {
		eventEngine.unbindEvent(self.widget, WIDGET_AFTER_MOVE_ROW_EVENT_NAME, afterMoveRowBoundEventIndex);
	}


	var onCloseSwipeBoundEventIndex = eventEngine.bindEvent(this.widget, WIDGET_ON_CLOSE_SWIPE_EVENT_NAME, setCurrentWidgetAndList);
	function setCurrentWidgetAndList() {
		self.widget.currentWidget = self.widget;
		self.widget.list();
	}

	$scope.$on("$destroy", unbindOnCloseSwipeListener);
	function unbindOnCloseSwipeListener() {
		eventEngine.unbindEvent(self.widget, WIDGET_ON_CLOSE_SWIPE_EVENT_NAME, onCloseSwipeBoundEventIndex);
	}

	var buildChangeModeFunction = function(widget, functionProperty, template) {
		return function() {
			widget[functionProperty]();
			widget.childTemplate = template;
		};
	};

	if(!this.widget.innerEdit){
		this.widget.innerEdit = this.widget.edit;
		this.widget.edit = buildChangeModeFunction(this.widget, "innerEdit", formTemplate);
	}

	if(!this.widget.innerList){
		this.widget.innerList = this.widget.list;
		this.widget.list = buildChangeModeFunction(this.widget, "innerList", formTemplate);
	}

	if(!this.widget.innerView){
		this.widget.innerView = this.widget.view;
		this.widget.view = buildChangeModeFunction(this.widget, "innerView", viewTemplate);
	}

	self.widget.activate();
}